/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.geometry;

/**
 * Utility class for polygon operations.
 */
public class Polygon {


    /**
     * Checks if point is on the right side of the line.
     * @param point point to check
     * @param lineP1 line start point
     * @param lineP2 line end point
     * @return true if point is on the right side of the line
     */
    private static boolean intersectsLine(final Point2D point, Point2D lineP1,
                                          Point2D lineP2) {

        // Sort line points by y coordinate.
        if (lineP1.y > lineP2.y) {
            final Point2D tmp = lineP1;
            lineP1 = lineP2;
            lineP2 = tmp;
        }

        // Check if point is within line y range.
        if (point.y < lineP1.y || point.y > lineP2.y)
            return false;

        // Check if point is on the line.
        final double xp = lineP2.x - lineP1.x;
        final double yp = lineP2.y - lineP1.y;

        final double crossX = lineP1.x + ((xp * (point.y - lineP1.y)) / yp);

        return point.x >= crossX;
    }

    public static boolean pointWithinPolygon(final Point2D point,
                                             final Point2D p1, final Point2D p2, final Point2D p3) {

        int intersectionCount = 0;

        if (intersectsLine(point, p1, p2))
            intersectionCount++;

        if (intersectsLine(point, p2, p3))
            intersectionCount++;

        if (intersectsLine(point, p3, p1))
            intersectionCount++;

        return intersectionCount == 1;

    }

}
