/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.gui.humaninput;

import eu.svjatoslav.sixth.e3d.gui.ViewPanel;

import java.util.Stack;

public class KeyboardFocusStack {

    private final ViewPanel viewPanel;
    private final WorldNavigationUserInputTracker defaultInputHandler = new WorldNavigationUserInputTracker();
    private final Stack<KeyboardInputHandler> inputHandlers = new Stack<>();
    private KeyboardInputHandler currentUserInputHandler;

    public KeyboardFocusStack(final ViewPanel viewPanel) {
        this.viewPanel = viewPanel;
        pushFocusOwner(defaultInputHandler);
    }

    public KeyboardInputHandler getCurrentFocusOwner() {
        return currentUserInputHandler;
    }

    public void popFocusOwner() {
        if (currentUserInputHandler != null)
            currentUserInputHandler.focusLost(viewPanel);

        if (inputHandlers.isEmpty())
            return;

        currentUserInputHandler = inputHandlers.pop();
        currentUserInputHandler.focusReceived(viewPanel);
    }

    public boolean pushFocusOwner(final KeyboardInputHandler newInputHandler) {
        boolean updateNeeded = false;

        if (currentUserInputHandler == newInputHandler)
            return false;

        if (currentUserInputHandler != null) {
            updateNeeded = currentUserInputHandler.focusLost(viewPanel);
            inputHandlers.push(currentUserInputHandler);
        }

        currentUserInputHandler = newInputHandler;
        updateNeeded |= currentUserInputHandler.focusReceived(viewPanel);

        return updateNeeded;
    }
}
