/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.texturedpolygon;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;

import static java.lang.Math.abs;

public class PolygonBorderInterpolator implements
        Comparable<PolygonBorderInterpolator> {

    // on-screen coordinates
    Point2D onScreenPoint1;
    Point2D onScreenPoint2;

    double distanceFromY1;
    private int onScreenHeight;
    private int onScreenWidth;
    private int onscreenAbsoluteHeight;
    private double textureWidth;
    private double textureHeight;
    // texture coordinates
    private Point2D texturePoint1;
    private Point2D texturePoint2;


    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;

        return o instanceof PolygonBorderInterpolator && compareTo((PolygonBorderInterpolator) o) == 0;
    }

    @Override
    public int hashCode() {
        int result = onScreenWidth;
        result = 31 * result + onscreenAbsoluteHeight;
        return result;
    }

    @Override
    public int compareTo(final PolygonBorderInterpolator otherInterpolator) {
        if (onscreenAbsoluteHeight < otherInterpolator.onscreenAbsoluteHeight)
            return 1;
        if (onscreenAbsoluteHeight > otherInterpolator.onscreenAbsoluteHeight)
            return -1;

        if (onScreenWidth < otherInterpolator.onScreenWidth)
            return 1;
        if (onScreenWidth > otherInterpolator.onScreenWidth)
            return -1;

        return 0;
    }

    public boolean containsY(final int y) {

        if (onScreenPoint1.y < onScreenPoint2.y) {
            if (y >= onScreenPoint1.y)
                return y <= onScreenPoint2.y;
        } else if (y >= onScreenPoint2.y)
            return y <= onScreenPoint1.y;

        return false;
    }

    public double getTX() {

        if (onScreenHeight == 0)
            return (texturePoint2.x + texturePoint1.x) / 2d;

        return texturePoint1.x + ((textureWidth * distanceFromY1) / onScreenHeight);
    }

    public double getTY() {

        if (onScreenHeight == 0)
            return (texturePoint2.y + texturePoint1.y) / 2d;

        return texturePoint1.y + ((textureHeight * distanceFromY1) / onScreenHeight);
    }

    public int getX() {

        if (onScreenHeight == 0)
            return (int) ((onScreenPoint2.x + onScreenPoint1.x) / 2d);

        return (int) (onScreenPoint1.x + ((onScreenWidth * distanceFromY1) / onScreenHeight));
    }

    public void setCurrentY(final int y) {
        distanceFromY1 = y - onScreenPoint1.y;
    }

    public void setPoints(final Point2D onScreenPoint1, final Point2D onScreenPoint2,
                          final Point2D texturePoint1, final Point2D texturePoint2) {

        this.onScreenPoint1 = onScreenPoint1;
        this.onScreenPoint2 = onScreenPoint2;
        this.texturePoint1 = texturePoint1;
        this.texturePoint2 = texturePoint2;

        onScreenHeight = (int) (onScreenPoint2.y - onScreenPoint1.y);
        onScreenWidth = (int) (onScreenPoint2.x - onScreenPoint1.x);
        onscreenAbsoluteHeight = abs(onScreenHeight);

        textureWidth = texturePoint2.x - texturePoint1.x;
        textureHeight = texturePoint2.y - texturePoint1.y;
    }

}
