/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.solid;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.renderer.raster.Color;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.solidpolygon.SolidPolygon;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;

public class SolidPolygonRectangularBox extends AbstractCompositeShape {

    /**
     * Draws solid rectangular box between using center and size.
     *
     * @param center center of the box
     * @param size   size of the box
     * @param color  box color
     */
    public SolidPolygonRectangularBox(final Point3D center, final double size, final Color color) {
        this(//
                new Point3D(center.x - (size / 2), center.y - (size / 2), center.z - (size / 2)), //
                new Point3D(center.x + (size / 2), center.y + (size / 2), center.z + (size / 2)), //
                color);
    }

    /**
     * Draws solid rectangular box between 2 given points in 3D space.
     */
    public SolidPolygonRectangularBox(final Point3D p1, final Point3D p7, final Color color) {
        super();

        final Point3D p2 = new Point3D(p7.x, p1.y, p1.z);
        final Point3D p3 = new Point3D(p7.x, p1.y, p7.z);
        final Point3D p4 = new Point3D(p1.x, p1.y, p7.z);

        final Point3D p5 = new Point3D(p1.x, p7.y, p1.z);
        final Point3D p6 = new Point3D(p7.x, p7.y, p1.z);
        final Point3D p8 = new Point3D(p1.x, p7.y, p7.z);

        addShape(new SolidPolygon(p1, p2, p3, color));
        addShape(new SolidPolygon(p1, p4, p3, color));

        addShape(new SolidPolygon(p5, p6, p7, color));
        addShape(new SolidPolygon(p5, p8, p7, color));

        addShape(new SolidPolygon(p4, p3, p8, color));
        addShape(new SolidPolygon(p3, p7, p8, color));

        addShape(new SolidPolygon(p1, p5, p4, color));
        addShape(new SolidPolygon(p4, p5, p8, color));

        addShape(new SolidPolygon(p1, p2, p5, color));
        addShape(new SolidPolygon(p2, p6, p5, color));

        addShape(new SolidPolygon(p2, p6, p3, color));
        addShape(new SolidPolygon(p6, p7, p3, color));
    }

}
