/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.slicer;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.math.Vertex;

import java.util.Comparator;

public class BorderLine implements Comparator<BorderLine> {

    public final int count;
    final Vertex c1;
    final Vertex c2;

    public BorderLine(final Vertex c1, final Vertex c2,
                      final int count) {
        this.c1 = c1;
        this.c2 = c2;
        this.count = count;
    }

    @Override
    public int compare(final BorderLine o1, final BorderLine o2) {
        return Double.compare(o1.getLength(), o2.getLength());
    }

    public double getLength() {
        return c1.coordinate.getDistanceTo(c2.coordinate);
    }

    public Vertex getMiddlePoint() {
        return new Vertex(
                new Point3D().computeMiddlePoint(c1.coordinate, c2.coordinate),
                new Point2D().setToMiddle(c1.textureCoordinate, c2.textureCoordinate));
    }
}
