/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.slicer;

import eu.svjatoslav.sixth.e3d.math.Vertex;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.texturedpolygon.TexturedPolygon;

import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.sort;

public class Slicer {

    /**
     * Maximum distance between two points.
     * If the distance is greater than this value, the polygon will be sliced.
     * Otherwise, it will be added to the result.
     */
    private final double maxDistance;

    /**
     * Result of slicing.
     */
    private final List<TexturedPolygon> result = new ArrayList<>();

    public Slicer(final double maxDistance) {
        this.maxDistance = maxDistance;
    }

    private void considerSlicing(final Vertex c1,
                                 final Vertex c2,
                                 final Vertex c3,
                                 final TexturedPolygon originalPolygon) {

        final BorderLine[] lines = new BorderLine[]{
                new BorderLine(c1, c2, 1), new BorderLine(c2, c3, 2),
                new BorderLine(c3, c1, 3)};

        sort(lines, lines[0]);

        final BorderLine longestLine = lines[2];

        if (longestLine.getLength() < maxDistance) {
            final TexturedPolygon polygon = new TexturedPolygon(c1, c2, c3,
                    originalPolygon.texture);

            polygon.setMouseInteractionController(originalPolygon.mouseInteractionController);

            getResult().add(polygon);
            return;
        }

        final Vertex middle = longestLine.getMiddlePoint();

        switch (longestLine.count) {
            case 1:
                considerSlicing(c1, middle, c3, originalPolygon);
                considerSlicing(middle, c2, c3, originalPolygon);
                return;
            case 2:
                considerSlicing(c1, c2, middle, originalPolygon);
                considerSlicing(middle, c3, c1, originalPolygon);
                return;
            case 3:
                considerSlicing(c1, c2, middle, originalPolygon);
                considerSlicing(middle, c2, c3, originalPolygon);
        }

    }

    public List<TexturedPolygon> getResult() {
        return result;
    }

    public void slice(final TexturedPolygon originalPolygon) {

        considerSlicing(
                originalPolygon.coordinates[0],
                originalPolygon.coordinates[1],
                originalPolygon.coordinates[2],
                originalPolygon);
    }

}
